@@ -3,7 +3,8 @@ import os
 import re
 import tempfile
 import shutil
-from ufoLib import UFOReader, UFOWriter
+from fontTools.misc.py23 import basestring
+from fontTools.ufoLib import UFOReader, UFOWriter, UFOLibError, UFOFileStructure
 from defcon.objects.base import BaseObject
 from defcon.objects.layerSet import LayerSet
 from defcon.objects.info import Info
@@ -16,6 +17,10 @@ from defcon.objects.dataSet import DataSet
 from defcon.objects.guideline import Guideline
 from defcon.tools.notifications import NotificationCenter
 from functools import partial
+import logging
+
+
+logger = logging.getLogger(__name__)
 
 
 class Font(BaseObject):
@@ -113,6 +118,7 @@ class Font(BaseObject):
 
         self._path = path
         self._ufoFormatVersion = None
+        self._ufoFileStructure = None
 
         self._kerning = None
         self._info = None
@@ -134,13 +140,19 @@ class Font(BaseObject):
         self._dirty = False
 
         if path:
-            reader = UFOReader(self._path)
+            if not isinstance(path, basestring) and not hasattr(path, "__fspath__"):
+                raise TypeError(
+                    "invalid path: expected string or os.PathLike, found %s"
+                    % type(path).__name__
+                )
+            reader = UFOReader(self._path, validate=self.ufoLibReadValidate)
             self._ufoFormatVersion = reader.formatVersion
+            self._ufoFileStructure = reader.fileStructure
             # go ahead and load the layers
             self._layers.disableNotifications()
             layerNames = reader.getLayerNames()
             for layerName in layerNames:
-                glyphSet = reader.getGlyphSet(layerName)
+                glyphSet = reader.getGlyphSet(layerName, validateRead=self._layers.ufoLibReadValidate, validateWrite=self._layers.ufoLibWriteValidate)
                 layer = self._layers.newLayer(layerName, glyphSet=glyphSet)
                 layer.dirty = False
                 self._beginSelfLayerNotificationObservation(layer)
@@ -252,16 +264,14 @@ class Font(BaseObject):
 
     def _set_path(self, path):
         # XXX: this needs to be reworked for layers
+        if not isinstance(path, basestring) and not hasattr(path, "__fspath__"):
+            raise TypeError(
+                "invalid path: expected string or os.PathLike, found %s"
+                % type(path).__name__
+            )
         # the file must already exist
         assert os.path.exists(path)
-        # the glyphs directory must already exist
-        glyphsDir = os.path.join(path, "glyphs")
-        assert os.path.exists(glyphsDir)
-        # set the internal reference
         self._path = path
-        # set the glyph set reference
-        if self._glyphSet is not None:
-            self._glyphSet.dirName = glyphsDir
 
     path = property(_get_path, _set_path, doc="The location of the file on disk. Setting the path should only be done when the user has moved the file in the OS interface. Setting the path is not the same as a save operation.")
 
@@ -270,6 +280,19 @@ class Font(BaseObject):
 
     ufoFormatVersion = property(_get_ufoFormatVersion, doc="The UFO format version that will be used when saving. This is taken from a loaded UFO during __init__. If this font was not loaded from a UFO, this will return None until the font has been saved.")
 
+    def _get_ufoFileStructure(self):
+        return self._ufoFileStructure
+
+    ufoFileStructure = property(
+        _get_ufoFileStructure,
+        doc=(
+            "The UFO file structure that will be used when saving. "
+            "This is taken from a loaded UFO during __init__. "
+            "If this font was not loaded from a UFO, this will return None "
+            "until the font has been saved."
+        )
+    )
+
     def _get_kerningGroupConversionRenameMaps(self):
         return self._kerningGroupConversionRenameMaps
 
@@ -372,8 +395,8 @@ class Font(BaseObject):
                 self.disableNotifications()
                 #
                 self._info.disableNotifications()
-                reader = UFOReader(self._path)
-                reader.readInfo(self._info)
+                reader = UFOReader(self._path, validate=False)
+                reader.readInfo(self._info, validate=self._info.ufoLibReadValidate)
                 self._info.dirty = False
                 self._info.enableNotifications()
                 #
@@ -392,14 +415,15 @@ class Font(BaseObject):
         if hasattr(self, "_reader"):
             reader = self._reader
         else:
-            reader = UFOReader(self._path)
-        kerning = reader.readKerning()
-        groups = reader.readGroups()
+            reader = UFOReader(self._path, validate=False)
         # instantiate everything and store it if valid
         self._groups = self.instantiateGroups()
         self.beginSelfGroupsNotificationObservation()
         self._kerning = self.instantiateKerning()
         self.beginSelfKerningNotificationObservation()
+        # load data
+        kerning = reader.readKerning(validate=self._kerning.ufoLibReadValidate)
+        groups = reader.readGroups(validate=self._groups.ufoLibReadValidate)
         # Note: the incoming kerning data has not been validated.
         # Gremlins may be sneaking in through here.
         ## store groups
@@ -504,7 +528,7 @@ class Font(BaseObject):
             reader = None
             if self._path is not None:
                 self._features.disableNotifications()
-                reader = UFOReader(self._path)
+                reader = UFOReader(self._path, validate=False)
                 t = reader.readFeatures()
                 self._features.text = t
                 self._features.dirty = False
@@ -540,8 +564,8 @@ class Font(BaseObject):
             reader = None
             if self._path is not None:
                 self._lib.disableNotifications()
-                reader = UFOReader(self._path)
-                d = reader.readLib()
+                reader = UFOReader(self._path, validate=False)
+                d = reader.readLib(validate=self._lib.ufoLibReadValidate)
                 self._lib.update(d)
                 self._lib.enableNotifications()
             self._stampLibDataState(reader)
@@ -680,7 +704,14 @@ class Font(BaseObject):
         count += self.layers.getSaveProgressBarTickCount(formatVersion)
         return count
 
-    def save(self, path=None, formatVersion=None, removeUnreferencedImages=False, progressBar=None):
+    def save(
+        self,
+        path=None,
+        formatVersion=None,
+        removeUnreferencedImages=False,
+        progressBar=None,
+        structure=None,
+    ):
         """
         Save the font to **path**. If path is None, the path
         from the last save or when the font was first opened
@@ -696,12 +727,69 @@ class Font(BaseObject):
         Optionally, the UFO can be purged of unreferenced images
         during this operation. To do this, pass ``True`` as the
         value for the removeUnreferencedImages argument.
+
+        'structure' can be either None, "zip" or "package". If it's None,
+        the destination UFO will use the same structure as original, provided
+        that is compatible with any previous UFO at the output path.
+        If 'structure' is "zip" the UFO will be saved as compressed archive,
+        else it is saved as a regular folder or "package".
         """
-        saveAs = False
-        if path is not None and path != self._path:
-            saveAs = True
-        else:
+        isNewFont = self._path is None
+        if path is None:
+            if isNewFont:
+                from defcon.errors import DefconError
+                raise DefconError("Can't save new font without a 'path'")
+            # saving in-place to the same original path
             path = self._path
+            saveAs = False
+        else:
+            if not isinstance(path, basestring) and not hasattr(path, "__fspath__"):
+                raise TypeError(
+                    "invalid path: expected string or os.PathLike, found %s"
+                    % type(path).__name__
+                )
+            if isNewFont:
+                # saving a new font is always a 'saveAs' operation
+                saveAs = True
+            else:
+                # 'saveAs' if source and destination path are different
+                saveAs = not samepath(self._path, path)
+
+        # validate 'structure' argument
+        if structure is not None:
+            try:
+                structure = UFOFileStructure(structure)
+            except ValueError:
+                from defcon.errors import DefconError
+                raise DefconError(
+                    "'%s' is not a valid UFOFileStructure; choose between %s"
+                    % (structure, tuple(e.value for e in UFOFileStructure))
+                )
+        elif self._ufoFileStructure is not None:
+            # if structure is None, fall back to the same as when first loaded
+            structure = self._ufoFileStructure
+        else:
+            # if both None, default to "package" structure
+            structure = UFOFileStructure.PACKAGE
+
+        # if destination is an existing path, ensure matches the desired structure
+        isExistingOSPath = os.path.exists(path)
+        if isExistingOSPath:
+            try:
+                with UFOReader(path, validate=True) as reader:
+                    existingStructure = reader.fileStructure
+            except UFOLibError:
+                # destination is an existing file but not a valid UFO, we'll
+                # silently overwrite it. Perhaps we should blow up...
+                saveAs = True
+            if not saveAs and structure and structure is not existingStructure:
+                from defcon.errors import DefconError
+                raise DefconError(
+                    "Can't save font in-place with a different structure; "
+                    "expected %s, got %s"
+                    % (existingStructure.value, structure.value)
+                )
+
         # sanity checks on layer data before doing anything destructive
         assert self.layers.defaultLayer is not None
         if self.layers.defaultLayer.name != "public.default":
@@ -714,15 +802,39 @@ class Font(BaseObject):
         # otherwise fallback to 3
         elif formatVersion is None:
             formatVersion = 3
-        # if down-converting, use a temp directory
-        convertinginPlace = False
-        if path == self._path and formatVersion != self._ufoFormatVersion:
-            convertinginPlace = True
+        # if down-converting in-place or "saving as" to a pre-existing path,
+        # we first write to a temporary folder, then move to destination
+        overwritePath = None
+        if ((not saveAs and formatVersion != self._ufoFormatVersion) or
+                (saveAs and isExistingOSPath)):
             saveAs = True
+            overwritePath = path
             path = os.path.join(tempfile.mkdtemp(), "temp.ufo")
         try:
             # make a UFOWriter
-            writer = UFOWriter(path, formatVersion=formatVersion)
+            try:
+                writer = UFOWriter(
+                    path,
+                    formatVersion=formatVersion,
+                    validate=self.ufoLibWriteValidate,
+                    structure=structure,
+                )
+            except UFOLibError:
+                if overwritePath is None and isExistingOSPath:
+                    logger.exception("Invalid ufo found '%s', the existing ufo "
+                                     "will be removed. Save will be handled as "
+                                     "save-as.", path)
+                    saveAs = True
+                    overwritePath = path
+                    path = os.path.join(tempfile.mkdtemp(), "temp.ufo")
+                    writer = UFOWriter(
+                        path,
+                        formatVersion=formatVersion,
+                        validate=self.ufoLibWriteValidate,
+                        structure=structure,
+                    )
+                else:
+                    raise
             # if changing ufo format versions, flag all objects
             # as dirty so that they will be saved
             if self._ufoFormatVersion != formatVersion:
@@ -748,18 +860,27 @@ class Font(BaseObject):
                 self.saveImages(writer=writer, removeUnreferencedImages=removeUnreferencedImages, saveAs=saveAs, progressBar=progressBar)
                 self.saveData(writer=writer, saveAs=saveAs, progressBar=progressBar)
             self.layers.save(writer, saveAs=saveAs, progressBar=progressBar)
+            # we must close the writer's filesystem to actually create the zip;
+            # Note that calling writer.close() makes all the SubFS instances
+            # derived from it unusable
+            if writer.fileStructure is UFOFileStructure.ZIP:
+                writer.close()
             writer.setModificationTime()
-            if convertinginPlace:
-                shutil.rmtree(self._path)
-                shutil.move(path, self._path)
+            if overwritePath is not None:
+                if os.path.isfile(overwritePath):
+                    os.remove(overwritePath)
+                elif os.path.isdir(overwritePath):
+                    shutil.rmtree(overwritePath)
+                shutil.move(path, overwritePath)
         finally:
-            # if down converting in place, handle the temp
-            if convertinginPlace:
+            # if down converting in place or overwriting, handle the temp
+            if overwritePath is not None:
                 shutil.rmtree(os.path.dirname(path))
-                path = self._path
+                path = overwritePath
         # done
         self._path = path
         self._ufoFormatVersion = formatVersion
+        self._ufoFileStructure = writer.fileStructure
         self.dirty = False
 
     def _saveInfo(self, writer, saveAs=False, progressBar=None):
@@ -768,7 +889,7 @@ class Font(BaseObject):
             progressBar.update(text="Saving info...", increment=0)
         self.saveInfo(writer)
         self.info.dirty = False
-        self._stampInfoDataState(UFOReader(writer.path))
+        self._stampInfoDataState(writer)
         if progressBar is not None:
             progressBar.update()
 
@@ -777,7 +898,7 @@ class Font(BaseObject):
         Save info. This method should not be called externally.
         Subclasses may override this method to implement custom saving behavior.
         """
-        writer.writeInfo(self.info)
+        writer.writeInfo(self.info, validate=self.info.ufoLibWriteValidate)
 
     def _saveGroups(self, writer, saveAs=False, progressBar=None):
         # groups should always be saved
@@ -785,7 +906,7 @@ class Font(BaseObject):
             progressBar.update(text="Saving groups...", increment=0)
         self.saveGroups(writer)
         self.groups.dirty = False
-        self._stampGroupsDataState(UFOReader(writer.path))
+        self._stampGroupsDataState(writer)
         if progressBar is not None:
             progressBar.update()
 
@@ -794,7 +915,7 @@ class Font(BaseObject):
         Save groups. This method should not be called externally.
         Subclasses may override this method to implement custom saving behavior.
         """
-        writer.writeGroups(self.groups)
+        writer.writeGroups(self.groups, validate=self.groups.ufoLibWriteValidate)
 
     def _saveKerning(self, writer, saveAs=False, progressBar=None):
         if self.kerning.dirty or saveAs:
@@ -802,7 +923,7 @@ class Font(BaseObject):
                 progressBar.update(text="Saving kerning...", increment=0)
             self.saveKerning(writer)
             self.kerning.dirty = False
-            self._stampKerningDataState(UFOReader(writer.path))
+            self._stampKerningDataState(writer)
         if progressBar is not None:
             progressBar.update()
 
@@ -811,7 +932,7 @@ class Font(BaseObject):
         Save kerning. This method should not be called externally.
         Subclasses may override this method to implement custom saving behavior.
         """
-        writer.writeKerning(self.kerning)
+        writer.writeKerning(self.kerning, validate=self.kerning.ufoLibWriteValidate)
 
     def _saveFeatures(self, writer, saveAs=False, progressBar=None):
         if self.features.dirty or saveAs:
@@ -820,7 +941,7 @@ class Font(BaseObject):
             if self.features.text is not None:
                 self.saveFeatures(writer)
             self.features.dirty = False
-            self._stampFeaturesDataState(UFOReader(writer.path))
+            self._stampFeaturesDataState(writer)
         if progressBar is not None:
             progressBar.update()
 
@@ -829,7 +950,7 @@ class Font(BaseObject):
         Save features. This method should not be called externally.
         Subclasses may override this method to implement custom saving behavior.
         """
-        writer.writeFeatures(self.features.text)
+        writer.writeFeatures(self.features.text, validate=self.features.ufoLibWriteValidate)
 
     def _saveLib(self, writer, saveAs=False, progressBar=None):
         # lib should always be saved
@@ -837,7 +958,7 @@ class Font(BaseObject):
             progressBar.update(text="Saving lib...", increment=0)
         self.saveLib(writer)
         self.lib.dirty = False
-        self._stampLibDataState(UFOReader(writer.path))
+        self._stampLibDataState(writer)
         if progressBar is not None:
             progressBar.update()
 
@@ -852,7 +973,7 @@ class Font(BaseObject):
         libCopy = dict(self.lib)
         if writer.formatVersion == 1:
             self._convertToFormatVersion1RoboFabData(libCopy)
-        writer.writeLib(libCopy)
+        writer.writeLib(libCopy, validate=self.lib.ufoLibWriteValidate)
 
     def saveImages(self, writer, removeUnreferencedImages=False, saveAs=False, progressBar=None):
         """
@@ -900,7 +1021,7 @@ class Font(BaseObject):
 
     def _set_guidelines(self, value):
         self.clearGuidelines()
-        self.holdNotifications()
+        self.holdNotifications(note="Requested by Font._set_guidelines.")
         for guideline in value:
             self.appendGuideline(guideline)
         self.releaseHeldNotifications()
@@ -989,7 +1110,7 @@ class Font(BaseObject):
 
         This posts a *Font.Changed* notification.
         """
-        self.holdNotifications()
+        self.holdNotifications(note="Requested by Font.clearGuidelines.")
         for guideline in reversed(self._guidelines):
             self.removeGuideline(guideline)
         self.releaseHeldNotifications()
@@ -1083,14 +1204,14 @@ class Font(BaseObject):
 
     def _stampFontDataState(self, obj, fileName, reader=None):
         # font is not on disk
-        if self.path is None:
+        if self.path is None and reader is None:
             return
         # data has not been loaded
         if obj is None:
             return
         # make a reader if necessary
         if reader is None:
-            reader = UFOReader(self.path)
+            reader = UFOReader(self.path, validate=False)
         # get the mod time from the reader
         modTime = reader.getFileModificationTime(fileName)
         # file is not in the UFO
@@ -1165,7 +1286,7 @@ class Font(BaseObject):
         this decision up to you.
         """
         assert self.path is not None
-        reader = UFOReader(self.path)
+        reader = UFOReader(self.path, validate=self.ufoLibReadValidate)
         infoChanged = self._testInfoForExternalModifications(reader)
         kerningChanged = self._testKerningForExternalModifications(reader)
         groupsChanged = self._testGroupsForExternalModifications(reader)
@@ -1215,7 +1336,7 @@ class Font(BaseObject):
             return
         # make a reader if necessary
         if reader is None:
-            reader = UFOReader(self.path)
+            reader = UFOReader(self.path, validate=False)
         # get the mod time from the reader
         modTime = reader.getFileModificationTime(fileName)
         # file is not in the UFO
@@ -1253,13 +1374,13 @@ class Font(BaseObject):
         Reload the data in the :class:`Info` object from the
         fontinfo.plist file in the UFO.
         """
-        from ufoLib import deprecatedFontInfoAttributesVersion2
+        from fontTools.ufoLib import deprecatedFontInfoAttributesVersion2
         if self._info is None:
             obj = self.info
         else:
-            reader = UFOReader(self.path)
+            reader = UFOReader(self.path, validate=False)
             newInfo = Info()
-            reader.readInfo(newInfo)
+            reader.readInfo(newInfo, validate=self._info.ufoLibReadValidate)
             oldInfo = self._info
             for attr in dir(newInfo):
                 if attr in deprecatedFontInfoAttributesVersion2:
@@ -1295,8 +1416,8 @@ class Font(BaseObject):
         if self._kerning is None:
             obj = self.kerning
         else:
-            reader = UFOReader(self._path)
-            kerning = reader.readKerning()
+            reader = UFOReader(self._path, validate=False)
+            kerning = reader.readKerning(validate=self._kerning.ufoLibReadValidate)
             # Note: the incoming kerning data has not been validated.
             # Gremlins may be sneaking in through here.
             self._kerning.clear()
@@ -1311,8 +1432,8 @@ class Font(BaseObject):
         if self._groups is None:
             obj = self.groups
         else:
-            reader = UFOReader(self._path)
-            d = reader.readGroups()
+            reader = UFOReader(self._path, validate=False)
+            d = reader.readGroups(validate=self._groups.ufoLibReadValidate)
             self._groups.clear()
             self._groups.update(d)
             self._stampGroupsDataState(reader)
@@ -1325,7 +1446,7 @@ class Font(BaseObject):
         if self._features is None:
             obj = self.features
         else:
-            reader = UFOReader(self._path)
+            reader = UFOReader(self._path, validate=False)
             text = reader.readFeatures()
             self._features.text = text
             self._stampFeaturesDataState(reader)
@@ -1338,8 +1459,8 @@ class Font(BaseObject):
         if self._lib is None:
             obj = self.lib
         else:
-            reader = UFOReader(self._path)
-            d = reader.readLib()
+            reader = UFOReader(self._path, validate=False)
+            d = reader.readLib(validate=self._lib.ufoLibReadValidate)
             self._lib.clear()
             self._lib.update(d)
             self._stampLibDataState(reader)
@@ -1641,6 +1762,16 @@ class Font(BaseObject):
             setter(key, data[key])
 
 
+def samepath(p1, p2):
+    """Return True if p1 and p2 refer to the same path. That is, when both
+    are strings or os.PathLike objects, compare their absolute, case
+    insensitive representation.
+    """
+    return os.path.normcase(os.path.realpath(os.path.abspath(p1))) == (
+        os.path.normcase(os.path.realpath(os.path.abspath(p2)))
+    )
+
+
 if __name__ == "__main__":
     import doctest
     doctest.testmod()
