@@ -2,6 +2,7 @@ from __future__ import absolute_import
 import weakref
 from warnings import warn
 from fontTools.misc.py23 import basestring
+from fontTools.misc.arrayTools import unionRect
 from defcon.objects.base import BaseObject
 from defcon.objects.contour import Contour
 from defcon.objects.point import Point
@@ -11,7 +12,7 @@ from defcon.objects.lib import Lib
 from defcon.objects.guideline import Guideline
 from defcon.objects.image import Image
 from defcon.objects.color import Color
-from defcon.tools.representations import glyphBoundsRepresentationFactory, glyphControlPointBoundsRepresentationFactory
+from defcon.tools.representations import glyphAreaRepresentationFactory
 from defcon.pens.decomposeComponentPointPen import DecomposeComponentPointPen
 
 
@@ -88,12 +89,8 @@ class Glyph(BaseObject):
     beginRedoNotificationName = "Glyph.BeginRedo"
     endRedoNotificationName = "Glyph.EndRedo"
     representationFactories = {
-        "defcon.glyph.bounds" : dict(
-            factory=glyphBoundsRepresentationFactory,
-            destructiveNotifications=("Glyph.ContoursChanged", "Glyph.ComponentsChanged", "Glyph.ComponentBaseGlyphDataChanged")
-        ),
-        "defcon.glyph.controlPointBounds" : dict(
-            factory=glyphControlPointBoundsRepresentationFactory,
+        "defcon.glyph.area" : dict(
+            factory=glyphAreaRepresentationFactory,
             destructiveNotifications=("Glyph.ContoursChanged", "Glyph.ComponentsChanged", "Glyph.ComponentBaseGlyphDataChanged")
         )
     }
@@ -252,16 +249,36 @@ class Glyph(BaseObject):
 
     # bounds
 
+    def _getContourComponentBounds(self, attr):
+        bounds = None
+        subObjects = [contour for contour in self]
+        subObjects += [component for component in self.components]
+        for subObject in subObjects:
+            b = getattr(subObject, attr)
+            if b is not None:
+                if bounds is None:
+                    bounds = b
+                else:
+                    bounds = unionRect(bounds, b)
+        return bounds
+
     def _get_bounds(self):
-        return self.getRepresentation("defcon.glyph.bounds")
+        return self._getContourComponentBounds("bounds")
 
     bounds = property(_get_bounds, doc="The bounds of the glyph's outline expressed as a tuple of form (xMin, yMin, xMax, yMax).")
 
     def _get_controlPointBounds(self):
-        return self.getRepresentation("defcon.glyph.controlPointBounds")
+        return self._getContourComponentBounds("controlPointBounds")
 
     controlPointBounds = property(_get_controlPointBounds, doc="The control bounds of all points in the glyph. This only measures the point positions, it does not measure curves. So, curves without points at the extrema will not be properly measured.")
 
+    # area
+
+    def _get_area(self):
+        return self.getRepresentation("defcon.glyph.area")
+
+    area = property(_get_area, doc="The area of the glyph's outline.")
+
     # margins
 
     def _get_leftMargin(self):
@@ -449,7 +466,7 @@ class Glyph(BaseObject):
         """
         Draw the glyph with **pen**.
         """
-        from ufoLib.pointPen import PointToSegmentPen
+        from fontTools.pens.pointPen import PointToSegmentPen
         pointPen = PointToSegmentPen(pen)
         self.drawPoints(pointPen)
 
@@ -480,7 +497,7 @@ class Glyph(BaseObject):
         """
         Get the pen used to draw into this glyph.
         """
-        from ufoLib.pointPen import SegmentToPointPen
+        from fontTools.pens.pointPen import SegmentToPointPen
         return SegmentToPointPen(self.getPointPen())
 
     def getPointPen(self):
@@ -562,7 +579,7 @@ class Glyph(BaseObject):
         """
         assert contour not in self
         assert contour.glyph in (self, None), "This contour belongs to another glyph."
-        self.postNotification(notification="Glyph.ContourWillBeAdded")
+        self.postNotification(notification="Glyph.ContourWillBeAdded", data=dict(object=contour))
         if contour.glyph is None:
             identifiers = self._identifiers
             if contour.identifier is not None:
@@ -611,11 +628,41 @@ class Glyph(BaseObject):
 
         This posts a *Glyph.Changed* notification.
         """
-        self.holdNotifications()
+        self.holdNotifications(note="Requested by Glyph.clearContours.")
         for contour in reversed(self):
             self.removeContour(contour)
         self.releaseHeldNotifications()
 
+    def correctContourDirection(self, trueType=False, segmentLength=10):
+        """
+        Correct the direction of all contours in the glyph.
+
+        This posts a *Glyph.Changed* notification.
+        """
+        # set the contours to the same direction
+        for contour in self:
+            contour.clockwise = False
+        # sort the contours by area in reverse (i.e. largest first)
+        contours = sorted(self, key=lambda contour: -contour.area)
+        # build a tree of nested contours
+        tree = {}
+        for largeIndex, largeContour in enumerate(contours):
+            for smallContour in contours[largeIndex + 1:]:
+                if largeContour.contourInside(smallContour, segmentLength=segmentLength):
+                    if largeContour not in tree:
+                        tree[largeContour] = []
+                    tree[largeContour].append(smallContour)
+        # run through the tree, largest to smallest, flipping
+        # the direction of each contour nested within another contour
+        for largeContour in contours:
+            if largeContour in tree:
+                for smallContour in tree[largeContour]:
+                    smallContour.reverse()
+        # set to the opposite if needed
+        if trueType:
+            for contour in self:
+                contour.reverse()
+
     # ----------
     # Components
     # ----------
@@ -672,7 +719,7 @@ class Glyph(BaseObject):
         """
         assert component not in self._components
         assert component.glyph in (self, None), "This component belongs to another glyph."
-        self.postNotification(notification="Glyph.ComponentWillBeAdded")
+        self.postNotification(notification="Glyph.ComponentWillBeAdded", data=dict(object=component))
         if component.glyph is None:
             if component.identifier is not None:
                 identifiers = self._identifiers
@@ -711,7 +758,7 @@ class Glyph(BaseObject):
 
         This posts a *Glyph.Changed* notification.
         """
-        self.holdNotifications()
+        self.holdNotifications(note="Requested by Glyph.clearComponents.")
         for component in reversed(self._components):
             self.removeComponent(component)
         self.releaseHeldNotifications()
@@ -725,7 +772,7 @@ class Glyph(BaseObject):
         This posts *Glyph.ComponentsChanged*, *Glyph.ContoursChanged*
         and *Glyph.Changed* notifications.
         """
-        self.holdNotifications()
+        self.holdNotifications(note="Requested by Glyph.decomposeComponent.")
         layer = self.layer
         pointPen = DecomposeComponentPointPen(self, layer)
         self._decomposeComponent(component, layer, pointPen)
@@ -744,7 +791,7 @@ class Glyph(BaseObject):
         """
         if not self.components:
             return
-        self.holdNotifications()
+        self.holdNotifications(note="Requested by Glyph.decomposeAllComponents.")
         layer = self.layer
         pointPen = DecomposeComponentPointPen(self, layer)
         for component in self.components:
@@ -771,7 +818,7 @@ class Glyph(BaseObject):
 
     def _set_anchors(self, value):
         self.clearAnchors()
-        self.holdNotifications()
+        self.holdNotifications(note="Requested by Glyph._set_anchors.")
         for anchor in value:
             self.appendAnchor(anchor)
         self.releaseHeldNotifications()
@@ -858,7 +905,7 @@ class Glyph(BaseObject):
 
         This posts a *Glyph.Changed* notification.
         """
-        self.holdNotifications()
+        self.holdNotifications(note="Requested by Glyph.clearAnchors.")
         for anchor in reversed(self._anchors):
             self.removeAnchor(anchor)
         self.releaseHeldNotifications()
@@ -877,7 +924,7 @@ class Glyph(BaseObject):
 
     def _set_guidelines(self, value):
         self.clearGuidelines()
-        self.holdNotifications()
+        self.holdNotifications(note="Requested by Glyph._set_guidelines.")
         for guideline in value:
             self.appendGuideline(guideline)
         self.releaseHeldNotifications()
@@ -966,7 +1013,7 @@ class Glyph(BaseObject):
 
         This posts a *Glyph.Changed* notification.
         """
-        self.holdNotifications()
+        self.holdNotifications(note="Requested by Glyph.clearGuidelines.")
         for guideline in reversed(self._guidelines):
             self.removeGuideline(guideline)
         self.releaseHeldNotifications()
@@ -1167,7 +1214,7 @@ class Glyph(BaseObject):
 
         This posts a *Glyph.Changed* notification.
         """
-        self.holdNotifications()
+        self.holdNotifications(note="Requested by Glyph.clear.")
         self.clearContours()
         self.clearComponents()
         self.clearAnchors()
