@@ -1,6 +1,6 @@
 from __future__ import absolute_import
 import weakref
-from ufoLib import UFOReader
+from fontTools.ufoLib import UFOReader, UFOFileStructure
 from defcon.objects.base import BaseObject
 from defcon.objects.layer import Layer
 
@@ -283,6 +283,8 @@ class LayerSet(BaseObject):
                     newName = actionData["newName"]
                     if oldName in writer.layerContents:
                         writer.renameGlyphSet(oldName, newName)
+                        if newName == self.defaultLayer.name:
+                            writer.renameGlyphSet(newName, newName, defaultLayer=True)
                 elif action == "default":
                     newDefault = actionData["newDefault"]
                     oldDefault = actionData["oldDefault"]
@@ -299,12 +301,27 @@ class LayerSet(BaseObject):
                     # this will be handled by the creation of the glyph set
                     pass
         # save the layers
+
+        # The parent ZipFS is going to be closed inside Font.save, hence
+        # any operations on the GlyphSet's SubFS will fail after that.
+        # To prevent this, we need to reset each layer._glyphSet to None
+        # after saving, when the file structure is zip.
+        # By the time we finish saving a layer, all the glyph data has
+        # been loaded so this is ok. However, setting _glyphSet to None
+        # also means that the Layer.testForExternalChanges method will
+        # produce no effect when the file structure is zip.
+        # This is understandable because only one process at a time
+        # can write to a zip file.
+        isZip = writer.fileStructure is UFOFileStructure.ZIP
+
         if writer.formatVersion < 3:
             if progressBar is not None:
                 progressBar.update(text="Saving glyphs...", increment=0)
             layer = self.defaultLayer
-            glyphSet = writer.getGlyphSet(layerName=None, defaultLayer=True)
+            glyphSet = writer.getGlyphSet(layerName=None, defaultLayer=True, validateRead=self.ufoLibReadValidate, validateWrite=self.ufoLibWriteValidate)
             layer.save(glyphSet, saveAs=saveAs, progressBar=progressBar)
+            if isZip:
+                layer._glyphSet = None
             if progressBar is not None:
                 progressBar.update()
         else:
@@ -313,8 +330,10 @@ class LayerSet(BaseObject):
                     progressBar.update(text="Saving layer \"%s\"..." % layerName, increment=0)
                 layer = self[layerName]
                 isDefaultLayer = layer == self.defaultLayer
-                glyphSet = writer.getGlyphSet(layerName=layerName, defaultLayer=isDefaultLayer)
+                glyphSet = writer.getGlyphSet(layerName=layerName, defaultLayer=isDefaultLayer, validateRead=self.ufoLibReadValidate, validateWrite=self.ufoLibWriteValidate)
                 layer.save(glyphSet, saveAs=saveAs, progressBar=progressBar)
+                if isZip:
+                    layer._glyphSet = None
                 # this prevents us from saving when the color was deleted
                 #if layer.lib or layer.color:
                 glyphSet.writeLayerInfo(layer)
@@ -428,13 +447,13 @@ class LayerSet(BaseObject):
         """
         Reload the layers. This should not be called externally.
         """
-        reader = UFOReader(self.font.path)
+        reader = UFOReader(self.font.path, validate=self.font.ufoLibReadValidate)
         # handle the layers
         currentLayerOrder = self.layerOrder
         for layerName, l in layerData.get("layers", {}).items():
             # new layer
             if layerName not in currentLayerOrder:
-                glyphSet = reader.getGlyphSet(layerName)
+                glyphSet = reader.getGlyphSet(layerName, validateRead=self.ufoLibReadValidate, validateWrite=self.font.ufoLibWriteValidate)
                 self.newLayer(layerName, glyphSet)
             # get the layer
             layer = self[layerName]
@@ -495,13 +514,13 @@ class _StaticLayerInfoMaker(object):
         self.color = None
 
     def pack(self):
-        from ufoLib.plistlib import writePlistToString
+        from fontTools.misc import plistlib
         data = {}
         if self.lib:
             data["lib"] = self.lib
         if self.color is not None:
             data["color"] = self.color
-        return writePlistToString(data)
+        return plistlib.dumps(data)
 
 
 if __name__ == "__main__":
